@@ -40,6 +40,9 @@ struct _GVfsAfpVolumePrivate
   guint16 volume_id;
 };
 
+static void
+attention_cb (GVfsAfpConnection *conn, guint attention, GVfsAfpVolume *volume);
+
 static void
 g_vfs_afp_volume_init (GVfsAfpVolume *volume)
 {
@@ -53,7 +56,12 @@ g_vfs_afp_volume_init (GVfsAfpVolume *volume)
 static void
 g_vfs_afp_volume_finalize (GObject *object)
 {
-  /* TODO: Add deinitalization code here */
+  GVfsAfpVolume *volume;
+  GVfsAfpVolumePrivate *priv;
+
+  volume = G_VFS_AFP_VOLUME (object);
+  priv = volume->priv;
+  g_signal_handlers_disconnect_by_func (priv->conn, attention_cb, volume);
 
   G_OBJECT_CLASS (g_vfs_afp_volume_parent_class)->finalize (object);
 }
@@ -82,6 +90,7 @@ g_vfs_afp_volume_new (GVfsAfpServer *server, GVfsAfpConnection *conn)
 
   priv->server = server;
   priv->conn = conn;
+  g_signal_connect (priv->conn, "attention", G_CALLBACK (attention_cb), volume);
 
   return volume;
 }
@@ -2887,3 +2896,14 @@ g_vfs_afp_volume_read_from_fork_finish (GVfsAfpVolume  *volume,
   
   return TRUE;
 }
+
+static void
+attention_cb (GVfsAfpConnection *conn, guint attention, GVfsAfpVolume *volume)
+{
+  /* Respond to the server notification with FPGetVolParms as the spec
+   * suggests.  Some servers disconnect us if we don't. */
+  if (attention == AFP_ATTENTION_CODE_SERVER_NOTIFICATION)
+    g_vfs_afp_volume_get_parms (volume,
+                                AFP_VOLUME_BITMAP_VOL_ID_BIT,
+                                NULL, NULL, NULL);
+}
