@@ -1586,6 +1586,9 @@ set_info_from_stat (GVfsBackendSmb *backend,
 
   g_file_info_set_file_type (info, file_type);
   g_file_info_set_size (info, statbuf->st_size);
+  g_file_info_set_attribute_uint64 (info,
+                                    G_FILE_ATTRIBUTE_STANDARD_ALLOCATED_SIZE,
+                                    statbuf->st_blocks * G_GUINT64_CONSTANT (512));
 
   t.tv_sec = statbuf->st_mtime;
 #if defined (HAVE_STRUCT_STAT_ST_MTIMENSEC)
@@ -1915,7 +1918,6 @@ do_enumerate (GVfsBackend *backend,
   SMBCFILE *dir;
   char dirents[1024*4];
   struct smbc_dirent *dirp;
-  GList *files;
   GFileInfo *info;
   GString *uri;
   int uri_start_len;
@@ -1952,8 +1954,6 @@ do_enumerate (GVfsBackend *backend,
 
   while (TRUE)
     {
-      files = NULL;
-      
       res = smbc_getdents (op_backend->smb_context, dir, (struct smbc_dirent *)dirents, sizeof (dirents));
       if (res <= 0)
 	break;
@@ -1982,7 +1982,8 @@ do_enumerate (GVfsBackend *backend,
 		{
 		  info = g_file_info_new ();
 		  g_file_info_set_name (info, dirp->name);
-		  files = g_list_prepend (files, info);
+                  g_vfs_job_enumerate_add_info (job, info);
+                  g_object_unref (info);
 		}
 	      else
 		{
@@ -1992,7 +1993,8 @@ do_enumerate (GVfsBackend *backend,
 		    {
 		      info = g_file_info_new ();
 		      set_info_from_stat (op_backend, info, &st, dirp->name, matcher);
-		      files = g_list_prepend (files, info);
+                      g_vfs_job_enumerate_add_info (job, info);
+                      g_object_unref (info);
 		    }
 		}
 	    }
@@ -2001,13 +2003,6 @@ do_enumerate (GVfsBackend *backend,
 	  dirp = (struct smbc_dirent *) (((char *)dirp) + dirlen);
 	  res -= dirlen;
 	}
-      
-      if (files)
-	{
-	  files = g_list_reverse (files);
-	  g_vfs_job_enumerate_add_infos (job, files);
-	  g_list_free_full (files, g_object_unref);
-	}
     }
       
   res = smbc_closedir (op_backend->smb_context, dir);
